package net.continuumsecurity.bh2012;

import java.net.URL;
import java.util.Properties;

import org.apache.log4j.PropertyConfigurator;
import org.jbehave.core.configuration.Configuration;
import org.jbehave.core.configuration.MostUsefulConfiguration;
import org.jbehave.core.embedder.StoryControls;
import org.jbehave.core.i18n.LocalizedKeywords;
import org.jbehave.core.io.CodeLocations;
import org.jbehave.core.io.LoadFromRelativeFile;
import org.jbehave.core.io.StoryPathResolver;
import org.jbehave.core.io.UnderscoredCamelCaseResolver;
import org.jbehave.core.junit.JUnitStory;
import org.jbehave.core.model.ExamplesTableFactory;
import org.jbehave.core.parsers.RegexStoryParser;
import org.jbehave.core.reporters.Format;
import org.jbehave.core.reporters.StoryReporterBuilder;
import org.jbehave.core.steps.InstanceStepsFactory;
import org.jbehave.core.steps.ParameterConverters;
import org.jbehave.core.steps.ParameterConverters.ExamplesTableConverter;
import org.jbehave.core.steps.SilentStepMonitor;

public class BaseStory extends JUnitStory {
	public BaseStory() {
	PropertyConfigurator.configure("log4j.properties");
	String storyDir = "file:///" + System.getProperty("user.dir")
			+ "/src/main/java";
	StoryPathResolver storyPathResolver = new UnderscoredCamelCaseResolver(
			".story");

	Properties viewProperties = new Properties();
	viewProperties.put("decorateNonHtml", "true");
	URL codeLocation = CodeLocations.codeLocationFromURL(storyDir);
	StoryReporterBuilder srb = new StoryReporterBuilder()
			.withDefaultFormats().withViewResources(viewProperties)
			.withFormats(Format.CONSOLE, Format.TXT, Format.HTML)
			.withFailureTrace(false);
	//Setup parameters
	ParameterConverters parameterConverters = new ParameterConverters().addConverters(new ExamplesTableConverter(new ExamplesTableFactory(new LoadFromRelativeFile(codeLocation))));
    // factory to allow parameter conversion and loading from external resources (used by StoryParser too)
    ExamplesTableFactory examplesTableFactory = new ExamplesTableFactory(new LocalizedKeywords(), new LoadFromRelativeFile(codeLocation), parameterConverters);
    // add custom coverters
    parameterConverters.addConverters(new ExamplesTableConverter(examplesTableFactory));
    
	Configuration configuration = new MostUsefulConfiguration()
			.useStoryControls(
					new StoryControls().doDryRun(false)
							.doSkipScenariosAfterFailure(false)
							.doResetStateBeforeStory(true)
							)
			.useStoryLoader(new LoadFromRelativeFile(codeLocation))
			.useStoryReporterBuilder(srb)
			.useStoryPathResolver(storyPathResolver)
			.useStepMonitor(new SilentStepMonitor())
			.useStoryParser(
					new RegexStoryParser(examplesTableFactory))
			.useParameterConverters(parameterConverters);
	configuration.storyControls().useStoryMetaPrefix("story_").useScenarioMetaPrefix("scenario_");		
	useConfiguration(configuration);
	addSteps(new InstanceStepsFactory(configuration,new DemoWebAppSteps())
			.createCandidateSteps());

	configuredEmbedder().embedderControls()
			.doGenerateViewAfterStories(true)
			.doIgnoreFailureInStories(true)
			.doIgnoreFailureInView(false);
	}
}
