/*******************************************************************************
 *    BDD-Security, application security testing framework
 * 
 * Copyright (C) `2012 Stephen de Vries`
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see `<http://www.gnu.org/licenses/>`.
 ******************************************************************************/
package net.continuumsecurity.bh2012;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.containsString;

import org.jbehave.core.annotations.AfterStory;
import org.jbehave.core.annotations.Alias;
import org.jbehave.core.annotations.BeforeStory;
import org.jbehave.core.annotations.Given;
import org.jbehave.core.annotations.Named;
import org.jbehave.core.annotations.Then;
import org.jbehave.core.annotations.When;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.htmlunit.HtmlUnitDriver;

public class DemoWebAppSteps {
	WebDriver driver;
	
	
	@BeforeStory
	public void init() {
		driver = new HtmlUnitDriver();
	}
	
	@AfterStory
	public void tearDown() {
		driver.close();
	}
	
	@Given("the login page")
	public void gotoLoginPage() {
		driver.get("http://www.ispatula.com:8081/ispatula/shop/signonForm.do");
	}
	
	@When("the user logs in with username: $username and password: $password")
	@Alias("the user logs in with <username> and <password>")
	public void login(@Named("username") String username,@Named("password") String password) {
		driver.findElement(By.name("username")).sendKeys(username);
		driver.findElement(By.name("password")).sendKeys(password);
		driver.findElement(By.name("update")).click();
	}
	
	@Then("the word: $matchWord should be on the page") 
	@Alias("the <matchWord> should be on the page") 
	public void findWordInPage(@Named("matchWord") String matchWord){
		assertThat(driver.getPageSource(),containsString(matchWord));
	}
}
